@@ -15,6 +15,8 @@ try:
 except ImportError:
     raise RuntimeError("You must install salt package to use the salt backend")
 
+from typing import Any, Optional
+
 from testinfra.backend import base
 
 
@@ -22,23 +24,30 @@ class SaltBackend(base.BaseBackend):
     HAS_RUN_SALT = True
     NAME = "salt"
 
-    def __init__(self, host, *args, **kwargs):
+    def __init__(self, host: str, *args: Any, **kwargs: Any):
         self.host = host
-        self._client = None
+        self._client: Optional[salt.client.LocalClient] = None
         super().__init__(self.host, *args, **kwargs)
 
     @property
-    def client(self):
+    def client(self) -> salt.client.LocalClient:
         if self._client is None:
             self._client = salt.client.LocalClient()
         return self._client
 
-    def run(self, command, *args, **kwargs):
+    def run(self, command: str, *args: str, **kwargs: Any) -> base.CommandResult:
         command = self.get_command(command, *args)
         out = self.run_salt("cmd.run_all", [command])
-        return self.result(out["retcode"], command, out["stdout"], out["stderr"])
+        return self.result(
+            out["retcode"],
+            command,
+            out["stdout"].encode("utf8"),
+            out["stderr"].encode("utf8"),
+            stdout=out["stdout"],
+            stderr=out["stderr"],
+        )
 
-    def run_salt(self, func, args=None):
+    def run_salt(self, func: str, args: Any = None) -> Any:
         out = self.client.cmd(self.host, func, args or [])
         if self.host not in out:
             raise RuntimeError(
@@ -48,13 +57,13 @@ class SaltBackend(base.BaseBackend):
         return out[self.host]
 
     @classmethod
-    def get_hosts(cls, host, **kwargs):
+    def get_hosts(cls, host: str, **kwargs: Any) -> list[str]:
         if host is None:
             host = "*"
         if any(c in host for c in "@*[?"):
             client = salt.client.LocalClient()
             if "@" in host:
-                hosts = client.cmd(host, "test.true", expr_form="compound").keys()
+                hosts = client.cmd(host, "test.true", tgt_type="compound").keys()
             else:
                 hosts = client.cmd(host, "test.true").keys()
             if not hosts:
