@@ -15,6 +15,7 @@ import shutil
 import sys
 import tempfile
 import time
+from typing import AnyStr
 
 import pytest
 
@@ -173,7 +174,7 @@ class NagiosReporter:
         return ret
 
 
-class SpooledTemporaryFile(tempfile.SpooledTemporaryFile):
+class SpooledTemporaryFile(tempfile.SpooledTemporaryFile[AnyStr]):
     def __init__(self, *args, **kwargs):
         if "b" in kwargs.get("mode", "b"):
             self._out_encoding = kwargs.pop("encoding")
